;;File Version: 3.0
;;Last Modified: November 25, 2009
;;Modified by: JEC

PRO OCCRAD, ra, dec, time, radius, lon, ringoccphi, $
            FRING = FRING, f_peri, f_node, $
            STAR = STAR, SUN = SUN

;;+
;;
;; Computes ring plane intercept radii for a stellar occultation.
;;
;; << Inputs >>
;;
;; Time: a scalar or vector of ET values at which times the radii will
;; be computed.
;; ra, dec: J2000 coordinates of star at the time of the occultation (deg.),
;; unaberrated, but corrected for proper motion and parallax.
;;
;; << Outputs >>
;;
;; radius: a scalar or vector of ring plane radius values (km)
;; lon: ring plane longitudes (degrees).
;;
;; << Restrictions >>
;;
;; Uses the CSPICE DLM for IDL.
;;
;; << History >>
;;
;; Joshua Colwell
;; Edit of geometer_engine for speed. October 21, 2004.
;; replace single precision !RADEG and !DTOR with double precision.
;; May 23, 2005: correct aberration to match geometer_engine. Compute
;; longitudes. Rearrange parameters.
;; November 10, 2005: Add FRING keyword and f_peri and f_node output.
;; April 27, 2006: Fix bug when input time is a vector in F ring loop.
;; July 12, 2006: Add general relativity correction.
;; July 17, 2006: Add STAR keyword. This gets star info from file rather
;; than requiring RA and Dec input.
;; August 14, 2006: Make GR correction a Keyword option.
;; August 15, 2006: GR bug corrections from Dick French. Validated to < 1 mm
;; without GR in comparison with RGF. Validated to ~10 cm with GR correction.
;; Test case is Rev 013 Alp Sco occultation.
;; September 18, 2006: Add SUN keyword for solar occultations.
;; Feburary 7, 2007: fix bug in F ring frame per Nicole Albers.
;; May 7, 2008: Correct bug in proper motion correction.
;; April 3, 2009: Do GR correction automatically. Make separate loops
;; for F ring and regular rings to remove IF statement from within the
;; loop.
;; Take determination of solar position out of time-dependent loop to
;; speed up star occ calculations. Assuming movement of Sun is small
;; relative to size of solar disk. Move pole calculation out of loop,
;; fixed at value for initial time in occ, also to speed up
;; computation. Deleted unnecessary calculation of "zerolon" from
;; xzerolon.
;; November 25, 2009: Calculate ringoccphi, based on geometer_engine.
;; Verified with geometer on spot-check with Rev 11 Sig Sgr.
;;-

target = 'SATURN'
dtor = !DPI/180D0
radeg = 180D0/!DPI

CSPICE_BODN2C, target, targetid, found
IF NOT found THEN BEGIN
    stopstring = 'Target '+target+$
         ' not found. Stopping. Use .c to continue if you fix the problem.'
    STOP, stopstring
ENDIF

planetid = FIX(targetid/100)*100L+99L
CSPICE_BODC2N, planetid, planet, found
CSPICE_BODVAR, planetid, 'GM', gmsat
gmsat = gmsat[0]
iref = 'J2000'
scname = 'CASSINI'
sc_id = -82L

abcorr = 'NONE'
framestring = 'IAU_'+target
CSPICE_BODVAR, targetid, 'RADII', radii

origin = [0D, 0D, 0D]

ringplanerad = -1000D0

IF KEYWORD_SET(STAR) THEN BEGIN
    starfile = FILE_WHICH('GeometerStars.txt') 
    IF STRLEN(starfile) EQ 0 THEN BEGIN
        starfile = DIALOG_PICKFILE(TITLE = $
                                   'Select GeometerStars.txt')
    ENDIF
    OPENR, starlun, starfile, /GET_LUN
    namecheck = ''
    ra = 0D0
    dec = 0D0
    pmra = 0D0
    pmdec = 0D0
    parallax = 0D0
    REPEAT BEGIN
        READF, starlun, namecheck
        READF, starlun, ra
        READF, starlun, dec
        READF, starlun, pmra
        READF, starlun, pmdec
        READF, starlun, parallax
    ENDREP UNTIL STRCMP(namecheck, star, /FOLD_CASE) OR namecheck EQ 'END'
    IF namecheck EQ 'END' THEN BEGIN
        STOP,'could not find star in file.'
    ENDIF
    CLOSE, starlun
    FREE_LUN, starlun

    ;; Correct the star position for proper motion and for parallax.
    ;; Get number of years since 1991.25 epoch of Hipparcos
    ;; star catalog position:
    time_since_epoch = time[0]/3.155D7+8.75
    
    decstar = dec+pmdec*time_since_epoch/3.6D6
    rastar = ra+pmra/COS(dec*DTOR)*time_since_epoch/3.6D6
    
    dist_to_star = 1D0/(parallax*1D-3)*3.086D13
    CSPICE_RADREC, dist_to_star, rastar*DTOR, decstar*DTOR, starpos
    
    CSPICE_SPKEZR, 'CASSINI', time[0], 'J2000', $
                   'NONE', 'SSB', scstate, lt_corr
    scpos = scstate[0:2]
    starpos-=scpos
    
    starunit = starpos/CSPICE_VNORM(starpos)
    
    CSPICE_RECRAD, starunit, unitdist, rastar, decstar
    
    ra = rastar*RADEG
    dec = decstar*RADEG
ENDIF

ra_internal = ra*dtor
dec_internal = dec*dtor
CSPICE_RADREC, 1D0, ra_internal, dec_internal, boresight_inert

ntime = N_ELEMENTS(time)
IF ntime GT 1 THEN t0 = time[0] ELSE t0 = time

radius = DBLARR(ntime)
lon = DBLARR(ntime)
ringoccphi = DBLARR(ntime)
tt=0L

clight = cspice_clight()

IF KEYWORD_SET(SUN) THEN BEGIN
   PRINT,'Note: OCCRAD is using a constant position for the Sun.'
   CSPICE_SPKPOS, 'SUN', time[tt], iref, abcorr, 'CASSINI', $
                  sunrapos, lt_corr
   CSPICE_RECRAD, sunrapos, unit, ra_internal, dec_internal
   boresight_inert = sunrapos/CSPICE_VNORM(sunrapos)
ENDIF

;; Factor used in general relativity correction:
grfactor = 4D0*gmsat/(clight*clight)

;;***************************************************************************
;; Loop for ring geometry defined by Saturn pole vector:
IF NOT KEYWORD_SET(FRING) THEN BEGIN
;; Get the Saturn pole vector at the start time.
   sat_pole_body = [0D0, 0D0, 1D0]
   CSPICE_PXFORM, iref, 'IAU_SATURN', t0, inert2p
   CSPICE_MTXV, inert2p, sat_pole_body, pole
   CSPICE_RECRAD, pole, unitvec, pole_ra, pole_dec
   
;; Now convert RA and Dec to a Cartesian vector for the pole.
   CSPICE_RADREC, 1D0, pole_ra, pole_dec, pole
   
;; Convert the pole vector to a CSPICE plane using nvc2pl.
;; Define origin of our system at center of Saturn.
   CSPICE_NVC2PL, pole, 0D0, ringplane
   
;; Get the intersection of the Saturn equatorial plane
;; with the J2000 X-Y plane to get an inertial reference
;; direction for zero longitude for the rings:
   xzerolon = CROSSP([0D0,0D0,1D0], pole)
   xzerolon = xzerolon/CSPICE_VNORM(xzerolon) 
   
REPEAT BEGIN
;; Correction for aberration:
    CSPICE_SPKEZR, 'SATURN', time[tt], iref, 'NONE', 'SSB', satstatessb, temp
    boresight_aberrated = boresight_inert+satstatessb[3:5]/clight
    boresight = boresight_aberrated/CSPICE_VNORM(boresight_aberrated)

;;Get a vector from the planet center to the spacecraft:
    CSPICE_SPKPOS, target, time[tt], iref, abcorr, scname, scloc_i, lt_corr

;; Do general relativity correction (deflection of starlight by Saturn's
;; gravity):

    ;; This is a non-iterative solution to the general relativity
    ;; correction. Following Appendix A in French et al., 1993,
    ;; Icarus Vol. 103, p. 163, Equations A8-A10.
    ;; Get vector to point of closest approach to planet:
    CSPICE_NPEDLN, radii[0], radii[1], radii[2], scloc_i, $
                   boresight, nearpoint, rayheight
    nearpointmag = CSPICE_VNORM(nearpoint)
    neardist = nearpointmag+rayheight
    impact_param = -nearpoint/nearpointmag*neardist
    ;; Compute deflection assuming a spherical Saturn. Although this
    ;; is generally not a good assumption, the magnitude of the change
    ;; for spacecraft based occultations is much smaller than the measurement
    ;; resolution. Joe Spitale calculates 17 cm change due to J2 for the 
    ;; 2004 Xi(2) Ceti occultation, which is the occultation at the greatest
    ;; distance from Saturn.
    boresight += grfactor/(neardist*neardist)*impact_param
;    boresight /= CSPICE_VNORM(boresight)

;;Get ring plane intercept radius.
    CSPICE_INRYPL, -scloc_i, boresight, ringplane, nxpts, occpoint_i
    radius[tt] = CSPICE_VNORM(occpoint_i)
    lon[tt] = CSPICE_VSEP(occpoint_i,xzerolon)*RADEG
    IF TOTAL(pole*CROSSP(xzerolon,occpoint_i)) LT 0D0 THEN $
      lon[tt] = 360D0-lon[tt]
    
;; Get ringoccphi (clock angle between star look vector projected into
;; ring plane and occpoint intercept vector):
    occpoint_p = inert2p##occpoint_i
    boresight_p = inert2p##boresight
    occp_3vec = [occpoint_p[0], occpoint_p[1], 0D0]
    occp_3vec /= CSPICE_VNORM(occp_3vec)
    b_p_proj = [boresight_p[0], boresight_p[1], 0D0]
    b_p_proj /= CSPICE_VNORM(b_p_proj)
    vcross = CROSSP(occp_3vec, b_p_proj)
    ringoccphi[tt] = CSPICE_VSEP(occp_3vec, b_p_proj)*RADEG
    IF vcross[2] LT 0. THEN ringoccphi[tt] = 360D0-ringoccphi[tt]
    tt++
;stop,'2'
ENDREP UNTIL tt EQ ntime
;;***************************************************************************
ENDIF ELSE BEGIN
;; F Ring plane. Bosh et al. (2002) model.
REPEAT BEGIN
;; Get the Saturn pole vector at the start time.
    sat_pole_body = [0D0, 0D0, 1D0]
    CSPICE_PXFORM, iref, 'IAU_SATURN', t0, inert2p
    CSPICE_MTXV, inert2p, sat_pole_body, pole
    CSPICE_RECRAD, pole, unitvec, pole_ra, pole_dec

    IF KEYWORD_SET(SUN) THEN BEGIN
        CSPICE_SPKPOS, 'SUN', time[tt], iref, abcorr, 'CASSINI', $
                       sunrapos, lt_corr
        CSPICE_RECRAD, sunrapos, unit, ra_internal, dec_internal
        boresight_inert = sunrapos/CSPICE_VNORM(sunrapos)
    ENDIF

    ;; Bosh et al. (2002) F Ring model uses the following Saturn
    ;; pole. Need to use it to have the appropriate F Ring pole.
;        nyears = time/3.15576D7
;        sat_pole_ra = 40.59287D0*DTOR - 6.1772D-4*DTOR*nyears
;        sat_pole_dec = 83.53833D0*DTOR -6.42D-5*DTOR*nyears

    ;; Get a three-vector for the Saturn pole:
;        CSPICE_RADREC, 1D0, sat_pole_ra, sat_pole_dec, sat_pole
    
    ;; Or, use NAIF values:
    sat_pole_ra = pole_ra
    sat_pole_dec = pole_dec
    sat_pole = pole
    
    ;; Bosh et al. (2002) Table III, Fit #3:
    fring_inc = 6.5D-3*DTOR
    fring_node = (16.1*DTOR - 2.6876*DTOR*t0/86400D0) MOD (2*!DPI)
    IF fring_node LT 0D0 THEN fring_node += (2*!DPI)
    f_node = fring_node*RADEG
    ;; fring_node is the angle from the Saturn ascending node to
    ;; the F Ring ascending node. 
    
    fring_pericenter = (24.1*DTOR + 2.7001*DTOR*t0/86400D0) MOD $
                       (2*!DPI)
    f_peri = fring_pericenter*RADEG
    ;; fring_pericenter is the angle from the Saturn ascending node
    ;; to the pericenter of the F ring.
    
    ;; Get a three-vector that points to the ascending node
    ;; of Saturn:
    sat_node_vec = [-sat_pole[1], sat_pole[0], 0D0]
    
    CSPICE_TWOVEC, sat_pole, 3, sat_node_vec, 1, inert2sat_node
    ;; inert2sat_node rotates from J2000 to a Saturn frame where
    ;; the X axis is along the ascending node of Saturn. 
    ;; Now get the matrix to go from this Saturn node frame to the F
    ;; ring frame. The x axis of the F ring frame points to the
    ;; ascending node of the F ring.
    CSPICE_EUL2M, -fring_node, -fring_inc, 0D0, $
                  3L, 1L, 3L, sat2f
    
    CSPICE_MTXM, sat2f, inert2sat_node, inert2f
    
    CSPICE_MTXV, inert2f, sat_pole_body, pole
    CSPICE_RECRAD, pole, unitvec, pole_ra, pole_dec
    ;; pole is the pole of the F ring in the inertial frame, and
    ;; pole_ra and pole_dec are the RA and Dec of the F ring pole.

;; Now convert RA and Dec to a Cartesian vector for the pole.
    CSPICE_RADREC, 1D0, pole_ra, pole_dec, pole
    
;; Convert the pole vector to a CSPICE plane using nvc2pl.
;; Define origin of our system at center of Saturn.
    CSPICE_NVC2PL, pole, 0D0, ringplane
    
;; Get the intersection of the Saturn equatorial plane
;; with the J2000 X-Y plane to get an inertial reference
;; direction for zero longitude for the rings:
    xzerolon = CROSSP([0D0,0D0,1D0], pole)
    xzerolon = xzerolon/CSPICE_VNORM(xzerolon) 

;; Correction for aberration:
    CSPICE_SPKEZR, 'SATURN', time[tt], iref, 'NONE', 'SSB', satstatessb, temp
    boresight_aberrated = boresight_inert+satstatessb[3:5]/clight
    boresight = boresight_aberrated/CSPICE_VNORM(boresight_aberrated)

;;Get a vector from the planet center to the spacecraft:
    CSPICE_SPKPOS, target, time[tt], iref, abcorr, scname, scloc_i, lt_corr

;; Do general relativity correction (deflection of starlight by Saturn's
;; gravity):

    ;; This is a non-iterative solution to the general relativity
    ;; correction. Following Appendix A in French et al., 1993,
    ;; Icarus Vol. 103, p. 163, Equations A8-A10.
    ;; Get vector to point of closest approach to planet:
    CSPICE_NPEDLN, radii[0], radii[1], radii[2], scloc_i, $
                   boresight, nearpoint, rayheight
    nearpointmag = CSPICE_VNORM(nearpoint)
    neardist = nearpointmag+rayheight
    impact_param = -nearpoint/nearpointmag*neardist
    ;; Compute deflection assuming a spherical Saturn. Although this
    ;; is generally not a good assumption, the magnitude of the change
    ;; for spacecraft based occultations is much smaller than the measurement
    ;; resolution. Joe Spitale calculates 17 cm change due to J2 for the 
    ;; 2004 Xi(2) Ceti occultation, which is the occultation at the greatest
    ;; distance from Saturn.
    boresight += grfactor/(neardist*neardist)*impact_param
    boresight /= CSPICE_VNORM(boresight)

;;Get ring plane intercept radius.
    CSPICE_INRYPL, -scloc_i, boresight, ringplane, nxpts, occpoint_i
    radius[tt] = CSPICE_VNORM(occpoint_i)
    lon[tt] = CSPICE_VSEP(occpoint_i,xzerolon)*RADEG
    IF TOTAL(pole*CROSSP(xzerolon,occpoint_i)) LT 0D0 THEN $
      lon[tt] = 360D0-lon[tt]
    
    tt++
;stop,'2'
ENDREP UNTIL tt EQ ntime
ENDELSE ;; F ring

END





